@@ -42,6 +42,7 @@
 #include <libgnomeui/gnome-messagebox.h>
 #include <e-util/e-dialog-utils.h>
 #include <e-util/e-icon-factory.h>
+#include <e-util/e-util-private.h>
 #include <evolution-shell-component-utils.h>
 
 #include <camel/camel-url.h>
@@ -1168,6 +1169,22 @@ key_press_event(GtkWidget *widget, GdkEventKey *event)
         return FALSE;
 }
 
+static gint
+editor_key_press_event(GtkWidget *widget, GdkEventKey *event, CompEditor *editor)
+{
+        EAttachmentBar *bar = E_ATTACHMENT_BAR (widget);
+
+        if (event->keyval == GDK_Escape) {
+		commit_all_fields (editor);
+		
+		if (prompt_to_save_changes (editor, TRUE))
+			close_dialog (editor);
+
+                return TRUE;
+        }
+                                                                                
+        return FALSE;
+}
 /* Menu callbacks */
 static void
 menu_file_save_cb (BonoboUIComponent *uic,
@@ -1337,6 +1354,7 @@ setup_widgets (CompEditor *editor)
 	gtk_notebook_set_show_tabs (priv->notebook, FALSE);
 
 	g_signal_connect (editor, "delete_event", G_CALLBACK (delete_event_cb), editor);
+	g_signal_connect (editor, "key_press_event", G_CALLBACK (editor_key_press_event), editor);	
 
 	/*Attachments */
 	priv->attachment_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -1400,6 +1418,7 @@ comp_editor_init (CompEditor *editor)
 {
 	CompEditorPrivate *priv;
 	BonoboUIContainer *container;
+	char *xmlfile;
 
 	priv = g_new0 (CompEditorPrivate, 1);
 	editor->priv = priv;
@@ -1425,9 +1444,14 @@ comp_editor_init (CompEditor *editor)
 
 	bonobo_ui_component_freeze (editor->uic, NULL);
 
+	xmlfile = g_build_filename (EVOLUTION_UIDIR,
+				    "evolution-editor.xml",
+				    NULL);
 	bonobo_ui_util_set_ui (editor->uic, PREFIX,
-			       EVOLUTION_UIDIR "/evolution-editor.xml",
+			       xmlfile,
 			       "evolution-editor", NULL);
+	g_free (xmlfile);
+
 	e_pixmaps_update (editor->uic, pixmaps);
 	bonobo_ui_component_thaw (editor->uic, NULL);
 
