@@ -28,7 +28,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <glib.h>
 #include <libecal/e-cal-time-util.h>
-#include <e-util/e-time-utils.h>
+#include <libedataserver/e-time-utils.h>
 #include "calendar-config.h"
 #include "e-alarm-list.h"
 
@@ -471,14 +471,20 @@ get_alarm_string (ECalComponentAlarm *alarm)
 
 		if (dur) {
 			if (trigger.u.rel_duration.is_neg)
-				str = g_strdup_printf (_("%s %s before the start of the appointment"),
+				/*Translator: The first %s refers to the base, which would be actions like 
+				 * "Play a Sound". Second %s refers to the duration string e.g:"15 minutes"*/
+				 str = g_strdup_printf (_("%s %s before the start of the appointment"),
 						       base, dur);
 			else
+				/*Translator: The first %s refers to the base, which would be actions like 
+				 * "Play a Sound". Second %s refers to the duration string e.g:"15 minutes"*/
 				str = g_strdup_printf (_("%s %s after the start of the appointment"),
 						       base, dur);
 
 			g_free (dur);
 		} else
+			/*Translator: The %s refers to the base, which would be actions like 
+			 * "Play a sound" */ 
 			str = g_strdup_printf (_("%s at the start of the appointment"), base);
 
 		break;
@@ -625,7 +631,6 @@ e_alarm_list_iter_n_children (GtkTreeModel *tree_model,
 	EAlarmList *alarm_list = E_ALARM_LIST (tree_model);
 
 	g_return_val_if_fail (E_IS_ALARM_LIST (tree_model), -1);
-	g_return_val_if_fail (IS_VALID_ITER (alarm_list, iter), -1);
 
 	if (iter == NULL)
 		return g_list_length (alarm_list->list);
