@@ -36,25 +36,25 @@
 #include <time.h>
 #include <sys/stat.h>
 #include <gdk/gdkkeysyms.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkvscrollbar.h>
 #include <gtk/gtkwindow.h>
-#include <gal/widgets/e-gui-utils.h>
-#include <gal/e-table/e-table-memory-store.h>
-#include <gal/e-table/e-cell-checkbox.h>
-#include <gal/e-table/e-cell-toggle.h>
-#include <gal/e-table/e-cell-text.h>
-#include <gal/e-table/e-cell-combo.h>
-#include <gal/widgets/e-popup-menu.h>
-#include <widgets/misc/e-cell-date-edit.h>
+#include <misc/e-gui-utils.h>
+#include <table/e-table-memory-store.h>
+#include <table/e-cell-checkbox.h>
+#include <table/e-cell-toggle.h>
+#include <table/e-cell-text.h>
+#include <table/e-cell-combo.h>
+#include <misc/e-popup-menu.h>
+#include <misc/e-cell-date-edit.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-util.h>
 #include <e-util/e-categories-config.h>
 #include <e-util/e-dialog-utils.h>
+#include <e-util/e-util-private.h>
 
 #include <libecal/e-cal-time-util.h>
 #include "e-cal-model-calendar.h"
@@ -204,6 +204,7 @@ setup_e_table (ECalListView *cal_list_view)
 	ECell             *cell, *popup_cell;
 	GnomeCanvas       *canvas;
 	GtkStyle          *style;
+	gchar		  *etspecfile;
 
 	model = E_CAL_MODEL_CALENDAR (e_calendar_view_get_model (E_CALENDAR_VIEW (cal_list_view)));
 
@@ -265,11 +266,15 @@ setup_e_table (ECalListView *cal_list_view)
 
 	/* Create table view */
 
+	etspecfile = g_build_filename (EVOLUTION_ETSPECDIR,
+				       "e-cal-list-view.etspec",
+				       NULL);
 	cal_list_view->table_scrolled = E_TABLE_SCROLLED (
 		e_table_scrolled_new_from_spec_file (E_TABLE_MODEL (model),
 						     extras,
-						     EVOLUTION_ETSPECDIR "/e-cal-list-view.etspec",
+						     etspecfile,
 						     NULL));
+	g_free (etspecfile);
 
 	/* Make sure text is readable on top of our color coding */
 
