@@ -18,6 +18,7 @@
 #include <gtk/gtkdialog.h>
 #include <libgnomeui/gnome-dialog.h>
 #include <glade/glade.h>
+#include "e-util/e-util-private.h"
 #include "calendar-commands.h"
 #include "calendar-config.h"
 #include "tag-calendar.h"
@@ -106,14 +107,12 @@ ecal_event (ECalendarItem *calitem, gpointer user_data)
 static struct tm
 get_current_time (ECalendarItem *calitem, gpointer data)
 {
-	char *location;
 	icaltimezone *zone;
 	struct tm tmp_tm = { 0 };
 	struct icaltimetype tt;
 
 	/* Get the current timezone. */
-	location = calendar_config_get_timezone ();
-	zone = icaltimezone_get_builtin_timezone (location);
+	zone = calendar_config_get_icaltimezone ();
 
 	tt = icaltime_from_timet_with_zone (time (NULL), FALSE, zone);
 
@@ -205,6 +204,7 @@ goto_dialog (GnomeCalendar *gcal)
 	time_t start_time;
 	struct icaltimetype tt;
 	int b;
+	char *gladefile;
 
 	if (dlg) {
 		return;
@@ -213,7 +213,11 @@ goto_dialog (GnomeCalendar *gcal)
 	dlg = g_new0 (GoToDialog, 1);
 	
 	/* Load the content widgets */
-	dlg->xml = glade_xml_new (EVOLUTION_GLADEDIR "/goto-dialog.glade", NULL, NULL);
+	gladefile = g_build_filename (EVOLUTION_GLADEDIR,
+				      "goto-dialog.glade",
+				      NULL);
+	dlg->xml = glade_xml_new (gladefile, NULL, NULL);
+	g_free (gladefile);
 	if (!dlg->xml) {
 		g_message ("goto_dialog(): Could not load the Glade XML file!");
 		g_free (dlg);
