@@ -28,7 +28,10 @@
 
 #include "e-dialog-utils.h"
 
+#include <gdkconfig.h>
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
+#endif
 
 #include <gtk/gtkmain.h>
 #include <gtk/gtkplug.h>
@@ -42,6 +45,7 @@
 #include <gtk/gtkfilesel.h>
 #endif
 
+#include <gconf/gconf-client.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 
@@ -119,6 +123,7 @@ e_notice_with_xid (GdkNativeWindow parent, GtkMessageType type, const char *form
 }
 
 
+#ifdef GDK_WINDOWING_X11
 /* Tests whether or not an X Window is being managed by the
  * window manager.
  */
@@ -145,6 +150,8 @@ window_is_wm_toplevel (Display *display, Window window)
 	return FALSE;
 }
 
+#endif
+
 /**
  * e_dialog_set_transient_for:
  * @dialog: a dialog window
@@ -162,11 +169,12 @@ e_dialog_set_transient_for (GtkWindow *dialog,
 			    GtkWidget *parent_widget)
 {
 	GtkWidget *toplevel;
+#ifdef GDK_WINDOWING_X11
 	Window parent, root_ret, *children;
 	unsigned int numchildren;
 	Display *display;
 	Status status;
-
+#endif
 	g_return_if_fail (GTK_IS_WINDOW (dialog));
 	g_return_if_fail (GTK_IS_WIDGET (parent_widget));
 
@@ -179,7 +187,7 @@ e_dialog_set_transient_for (GtkWindow *dialog,
 					      GTK_WINDOW (toplevel));
 		return;
 	}
-
+#ifdef GDK_WINDOWING_X11
 	/* Find the top-level windowmanager-managed X Window */
 	display = GDK_WINDOW_XDISPLAY (parent_widget->window);
 	parent = GDK_WINDOW_XID (parent_widget->window);
@@ -192,6 +200,10 @@ e_dialog_set_transient_for (GtkWindow *dialog,
 	}
 
 	e_dialog_set_transient_for_xid (dialog, parent);
+#endif
+#ifdef GDK_WINDOWING_WIN32
+	g_warning ("Not implemented on Win32: e_dialog_set_transient_for() for plug windows");
+#endif
 }
 
 static void
@@ -250,6 +262,8 @@ save_ok (GtkWidget *widget, gpointer data)
 	char **filename = data;
 	const char *path;
 	int btn = GTK_RESPONSE_YES;
+	GConfClient *gconf = gconf_client_get_default();
+	char *dir;
 	
 	fs = gtk_widget_get_toplevel (widget);
 #ifdef USE_GTKFILECHOOSER
@@ -272,9 +286,14 @@ save_ok (GtkWidget *widget, gpointer data)
 		btn = gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 	}
-	
-	if (btn == GTK_RESPONSE_YES)
+
+	if (btn == GTK_RESPONSE_YES) {
+		dir = g_path_get_dirname (path);
+		gconf_client_set_string(gconf, "/apps/evolution/mail/save_dir", dir, NULL);
+		g_free (dir);
 		*filename = g_strdup (path);
+	}
+	g_object_unref(gconf);
 	
 	gtk_main_quit ();
 }
@@ -291,11 +310,20 @@ filechooser_response (GtkWidget *fc, gint response_id, gpointer data)
 #endif
 
 char *
-e_file_dialog_save (const char *title)
+e_file_dialog_save (const char *title, const char *fname)
 {
 	GtkWidget *selection;
 	char *filename = NULL;
+        char *dir, *gdir = NULL;
+        GConfClient *gconf;
+ 
+        gconf = gconf_client_get_default();
+        dir = gdir = gconf_client_get_string(gconf, "/apps/evolution/mail/save_dir", NULL);
+        g_object_unref(gconf);
 
+       if (dir == NULL)
+	       dir = (char *)g_get_home_dir();
+		    
 #ifdef USE_GTKFILECHOOSER
 	selection = gtk_file_chooser_dialog_new (title,
 						 NULL,
@@ -304,14 +332,17 @@ e_file_dialog_save (const char *title)
 						 GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 						 NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (selection), GTK_RESPONSE_ACCEPT);
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (selection), g_get_home_dir ());
-
+        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (selection), dir);
+ 
+        if (fname)
+	        gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER (selection), fname);
+     
 	g_signal_connect (G_OBJECT (selection), "response", G_CALLBACK (filechooser_response), &filename);
 #else
 	char *path;
 
 	selection = gtk_file_selection_new (title);
-	path = g_strdup_printf ("%s/", g_get_home_dir ());
+	path = g_strdup_printf ("%s/", dir);	
 	gtk_file_selection_set_filename (GTK_FILE_SELECTION (selection), path);
 	g_free (path);
 
@@ -324,6 +355,86 @@ e_file_dialog_save (const char *title)
 	gtk_main ();
 	
 	gtk_widget_destroy (GTK_WIDGET (selection));
+	g_free (gdir);
+	
+	return filename;
+}
+
+static void
+save_folder_ok (GtkWidget *widget, gpointer data)
+{
+	GtkWidget *fs;
+	char **filename = data;
+	const char *path;
+	GConfClient *gconf = gconf_client_get_default();
+	
+	fs = gtk_widget_get_toplevel (widget);
+#ifdef USE_GTKFILECHOOSER
+	path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (fs));
+#else
+	path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
+#endif
+	
+	gconf_client_set_string(gconf, "/apps/evolution/mail/save_dir", path, NULL);
+	g_object_unref(gconf);
+	*filename = g_strdup (path);
+	
+	gtk_main_quit ();
+}
+
+#ifdef USE_GTKFILECHOOSER
+static void
+folderchooser_response (GtkWidget *fc, gint response_id, gpointer data)
+{
+	if (response_id == GTK_RESPONSE_ACCEPT)
+		save_folder_ok (fc, data);
+	else
+		gtk_widget_destroy (fc);
+}
+#endif
+
+char *
+e_file_dialog_save_folder (const char *title)
+{
+	GtkWidget *selection;
+	char *path, *filename = NULL;
+	char *dir, *gdir = NULL;
+	GConfClient *gconf;
+
+	gconf = gconf_client_get_default();
+	dir = gdir = gconf_client_get_string(gconf, "/apps/evolution/mail/save_dir", NULL);
+	g_object_unref(gconf);
+
+	if (dir == NULL)
+		dir = (char *)g_get_home_dir();
+
+#ifdef USE_GTKFILECHOOSER
+	selection = gtk_file_chooser_dialog_new (title,
+						 NULL,
+						 GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+						 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						 GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+						 NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (selection), GTK_RESPONSE_ACCEPT);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (selection), dir);
+
+	g_signal_connect (G_OBJECT (selection), "response", G_CALLBACK (folderchooser_response), &filename);
+#else
+	selection = gtk_file_selection_new (title);
+	path = g_strdup_printf ("%s/", dir);
+	gtk_file_selection_set_filename (GTK_FILE_SELECTION (selection), path);
+	g_free (path);
+
+	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (selection)->ok_button), "clicked", G_CALLBACK (save_folder_ok), &filename);
+	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (selection)->cancel_button), "clicked", G_CALLBACK (gtk_main_quit), NULL);
+#endif
+	
+	gtk_widget_show (GTK_WIDGET (selection));
+	gtk_grab_add (GTK_WIDGET (selection));
+	gtk_main ();
+	
+	gtk_widget_destroy (GTK_WIDGET (selection));
+	g_free (gdir);
 	
 	return filename;
 }
