@@ -64,6 +64,7 @@
 #include "em-composer-utils.h"
 #include "em-format-quote.h"
 #include "em-account-editor.h"
+#include "e-attachment.h"
 
 static void emu_save_part_done (CamelMimePart *part, char *name, int done, void *data);
 
@@ -117,6 +118,44 @@ em_utils_prompt_user(GtkWindow *parent, const char *promptkey, const char *tag,
 	return button == GTK_RESPONSE_YES;
 }
 
+ 
+char *
+em_utils_temp_save_attachments (GSList *attachments)
+{
+        char *path = NULL;
+        GSList *selected;
+	
+	path = e_mkdtemp("evolution-attachments-XXXXXX");
+
+                for ( selected = attachments; selected != NULL; selected = selected->next) {
+                        const char *file_name;
+			char *safe_name = NULL;
+                       char *file_path;
+                        CamelMimePart *part = (CamelMimePart *)((EAttachment *)selected->data)->body;
+
+                        file_name = camel_mime_part_get_filename(part);
+                        if (file_name == NULL) {
+                                file_name = _("attachment");
+                        } else {
+				safe_name = g_strdup(file_name);
+				e_filename_make_safe(safe_name);
+				file_name = safe_name;
+			}
+			
+			file_path = g_build_filename (path, file_name, NULL);
+
+			if (!g_file_test(file_path, (G_FILE_TEST_EXISTS)))
+				mail_save_part(part, file_path, NULL, NULL);
+			else
+				g_warning ("Could not save %s. File already exists", file_path);
+
+			g_free (file_path);
+			g_free (safe_name);
+               }
+
+	return path;
+}
+
 /**
  * em_utils_uids_copy:
  * @uids: array of uids
@@ -226,7 +265,7 @@ em_utils_check_user_can_send_mail (GtkWidget *parent)
 	return TRUE;
 }
 
-/* Editing Filters/vFolders... */
+/* Editing Filters/Search Folders... */
 
 static GtkWidget *filter_editor = NULL;
 
@@ -250,10 +289,10 @@ em_filter_editor_response (GtkWidget *dialog, int button, gpointer user_data)
 	filter_editor = NULL;
 }
 
-static const char *em_filter_source_element_names[] = {
-	"incoming",
-	"outgoing",
-	NULL,
+static EMFilterSource em_filter_source_element_names[] = {
+	{ "incoming", },
+	{ "outgoing", },
+	{ 0 }
 };
 
 /**
@@ -286,7 +325,12 @@ em_utils_edit_filters (GtkWidget *parent)
 		e_error_run((GtkWindow *)parent, "mail:filter-load-error", ((RuleContext *)fc)->error, NULL);
 		return;
 	}
-	
+
+	if (em_filter_source_element_names[0].name == NULL) {
+		em_filter_source_element_names[0].name = _("Incoming");
+		em_filter_source_element_names[1].name = _("Outgoing");
+	}
+
 	filter_editor = (GtkWidget *) em_filter_editor_new (fc, em_filter_source_element_names);
 	if (parent != NULL)
 		e_dialog_set_transient_for ((GtkWindow *) filter_editor, parent);
@@ -356,14 +400,15 @@ emu_get_save_filesel (GtkWidget *parent, const char *title, const char *name, Gt
 }
 
 static void
-emu_update_save_path(const char *filename)
+emu_update_save_path(const char *filename, gboolean path)
 {
-	char *dir = g_path_get_dirname(filename);
+	char *dir = path ? (char *)filename : g_path_get_dirname(filename);
 	GConfClient *gconf = gconf_client_get_default();
 
 	gconf_client_set_string(gconf, "/apps/evolution/mail/save_dir", dir, NULL);
 	g_object_unref(gconf);
-	g_free(dir);
+	if (!path)
+		g_free(dir);
 }
 
 static gboolean
@@ -405,7 +450,7 @@ emu_save_part_response(GtkWidget *filesel, int response, CamelMimePart *part)
 		if (!emu_can_save((GtkWindow *)filesel, path))
 			return;
 
-		emu_update_save_path(path);
+		emu_update_save_path(path, FALSE);
 		/* FIXME: popup error if it fails? */
 		mail_save_part(part, path, NULL, NULL);
 	}
@@ -457,7 +502,7 @@ emu_save_parts_response (GtkWidget *filesel, int response, GSList *parts)
                 path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
 #endif
 
-                emu_update_save_path(path);
+                emu_update_save_path(path, TRUE);
 
                 for ( selected = parts; selected != NULL; selected = selected->next) {
                         const char *file_name;
@@ -575,7 +620,7 @@ emu_save_messages_response(GtkWidget *filesel, int response, struct _save_messag
 		if (!emu_can_save((GtkWindow *)filesel, path))
 			return;
 
-		emu_update_save_path(path);
+		emu_update_save_path(path, FALSE);
 		mail_save_messages(data->folder, data->uids, path, NULL, NULL);
 		data->uids = NULL;
 	}
@@ -1871,7 +1916,8 @@ em_utils_in_addressbook(CamelInternetAddress *iaddr)
 	time_t now;
 
 	/* TODO: check all addresses? */
-	if (!camel_internet_address_get(iaddr, 0, NULL, &addr))
+	if (iaddr == NULL
+	    || !camel_internet_address_get(iaddr, 0, NULL, &addr))
 		return FALSE;
 
 	pthread_mutex_lock(&emu_addr_lock);
