@@ -174,7 +174,7 @@ org_gnome_exchange_settings(EPlugin *epl, EConfigHookItemFactoryData *data)
 	CamelURL *url;
 	const char *source_url;
 	char *message = NULL, *txt = NULL, *oof_message;
-	gboolean oof_state;
+	gboolean oof_state = FALSE;
 
 	GtkVBox *vbox_settings;
 
@@ -232,7 +232,7 @@ org_gnome_exchange_settings(EPlugin *epl, EConfigHookItemFactoryData *data)
 
 	/* See if oof info found already */
 	
-	if (!exchange_oof_get (account, &oof_state, &message)) {
+	if (account && !exchange_oof_get (account, &oof_state, &message)) {
 
 		e_error_run (NULL, ERROR_DOMAIN ":state-read-error", NULL);
 
@@ -407,7 +407,9 @@ print_error (const char *owa_url, E2kAutoconfigResult result)
 			break;
 
 		case E2K_AUTOCONFIG_CANT_BPROPFIND:
-			e_error_run (NULL, ERROR_DOMAIN ":connect-exchange-error", NULL);
+			e_error_run (NULL, ERROR_DOMAIN ":connect-exchange-error", 
+				     "http://support.novell.com/cgi-bin/search/searchtid.cgi?/ximian/ximian328.html",
+				     NULL);
 			break;
 
 		case E2K_AUTOCONFIG_EXCHANGE_5_5:
@@ -692,11 +694,10 @@ set_oof_info ()
 
 	account = exchange_operations_get_exchange_account ();
 
-	if (!exchange_oof_set (account, oof_data->state, oof_data->message)) {
+	if (account && !exchange_oof_set (account, oof_data->state, oof_data->message)) {
 
 		e_error_run (NULL, ERROR_DOMAIN ":state-update-error", NULL);
 	}
-
 }
 
 static void
