@@ -36,13 +36,14 @@
 #include <libedataserver/e-time-utils.h>
 #include <libedataserverui/e-source-option-menu.h>
 #include <libecal/e-cal.h>
+#include <libecal/e-cal-time-util.h>
 #include <gtkhtml/gtkhtml-embedded.h>
 #include <mail/em-format-hook.h>
 #include <mail/em-format-html.h>
 #include <e-util/e-account-list.h>
 #include <e-util/e-icon-factory.h>
-#include <e-util/e-time-utils.h>
 #include <e-util/e-gtk-utils.h>
+#include <e-util/e-util.h>
 #include <calendar/gui/itip-utils.h>
 #include "itip-view.h"
 
@@ -112,6 +113,9 @@ struct _ItipViewPrivate {
 	GtkWidget *rsvp_comment_entry;
 	gboolean rsvp_show;
 
+	GtkWidget *recur_box;
+	GtkWidget *recur_check;
+
 	GtkWidget *update_box;
 	GtkWidget *update_check;
 	gboolean update_show;
@@ -143,7 +147,7 @@ format_date_and_time_x		(struct tm	*date_tm,
 	
 	/* Calculate a normalized "tomorrow" */
 	tomorrow_tm = *current_tm;
-	if (tomorrow_tm.tm_mday == time_days_in_month (date_tm->tm_year + 1900, date_tm->tm_mon - 1)) {
+	if (tomorrow_tm.tm_mday == time_days_in_month (date_tm->tm_year + 1900, date_tm->tm_mon)) {
 		tomorrow_tm.tm_mday = 1;
 		if (tomorrow_tm.tm_mon == 11) {
 			tomorrow_tm.tm_mon = 1;
@@ -157,8 +161,8 @@ format_date_and_time_x		(struct tm	*date_tm,
 
 	/* Calculate a normalized "next seven days" */
 	week_tm = *current_tm;
-	if (week_tm.tm_mday + 6 > time_days_in_month (date_tm->tm_year + 1900, date_tm->tm_mon - 1)) {
-		week_tm.tm_mday = (week_tm.tm_mday + 6) % time_days_in_month (date_tm->tm_year + 1900, date_tm->tm_mon - 1);
+	if (week_tm.tm_mday + 6 > time_days_in_month (date_tm->tm_year + 1900, date_tm->tm_mon)) {
+		week_tm.tm_mday = (week_tm.tm_mday + 6) % time_days_in_month (date_tm->tm_year + 1900, date_tm->tm_mon);
 		if (week_tm.tm_mon == 11) {
 			week_tm.tm_mon = 1;
 			week_tm.tm_year++;
@@ -689,9 +693,11 @@ static void
 set_buttons (ItipView *view) 
 {
 	ItipViewPrivate *priv;
-	
+	gboolean is_recur_set = FALSE;
+
 	priv = view->priv;
 
+	is_recur_set = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->recur_check));
 	gtk_container_foreach (GTK_CONTAINER (priv->button_box), (GtkCallback) gtk_widget_destroy, NULL);
 
 	/* Everything gets the open button */
@@ -703,9 +709,9 @@ set_buttons (ItipView *view)
 		set_one_button (view, "_Accept", GTK_STOCK_APPLY, ITIP_VIEW_RESPONSE_ACCEPT);
 		break;
 	case ITIP_VIEW_MODE_REQUEST:
-		set_one_button (view, "_Decline", GTK_STOCK_CANCEL, ITIP_VIEW_RESPONSE_DECLINE);
-		set_one_button (view, "_Tentative", GTK_STOCK_DIALOG_QUESTION, ITIP_VIEW_RESPONSE_TENTATIVE);
-		set_one_button (view, "_Accept", GTK_STOCK_APPLY, ITIP_VIEW_RESPONSE_ACCEPT);
+		set_one_button (view, is_recur_set ? "_Decline all" : "_Decline", GTK_STOCK_CANCEL, ITIP_VIEW_RESPONSE_DECLINE);
+		set_one_button (view, is_recur_set ? "_Tentative all" : "_Tentative", GTK_STOCK_DIALOG_QUESTION, ITIP_VIEW_RESPONSE_TENTATIVE);
+		set_one_button (view, is_recur_set ? "_Accept all" : "_Accept", GTK_STOCK_APPLY, ITIP_VIEW_RESPONSE_ACCEPT);
 		break;
 	case ITIP_VIEW_MODE_ADD:
 		set_one_button (view, "_Decline", GTK_STOCK_CANCEL, ITIP_VIEW_RESPONSE_DECLINE);
@@ -810,6 +816,20 @@ rsvp_toggled_cb (GtkWidget *widget, gpointer data)
 	gtk_widget_set_sensitive (priv->rsvp_comment_entry, rsvp);
 }
 
+static void
+recur_toggled_cb (GtkWidget *widget, gpointer data) 
+{
+	ItipView *view = data;
+	ItipViewPrivate *priv;
+	gboolean is_recur;
+	
+	priv = view->priv;
+	
+	is_recur = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->recur_check));
+	itip_view_set_mode (view, priv->mode);
+}
+
+
 static void
 itip_view_init (ItipView *view)
 {
@@ -961,6 +981,16 @@ itip_view_init (ItipView *view)
 	gtk_widget_show (priv->update_check);
 	gtk_box_pack_start (GTK_BOX (priv->update_box), priv->update_check, FALSE, FALSE, 0);
 
+	/* The recurrence check button */
+	priv->recur_box = gtk_vbox_new (FALSE, 12);
+	gtk_widget_show (priv->recur_box);
+	gtk_box_pack_start (GTK_BOX (vbox), priv->recur_box, FALSE, FALSE, 0);
+
+	priv->recur_check = gtk_check_button_new_with_mnemonic (_("A_pply to all instances"));
+	gtk_box_pack_start (GTK_BOX (priv->recur_box), priv->recur_check, FALSE, FALSE, 0);
+
+	g_signal_connect (priv->recur_check, "toggled", G_CALLBACK (recur_toggled_cb), view);
+
 	/* The buttons for actions */
 	priv->button_box = gtk_hbutton_box_new ();
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (priv->button_box), GTK_BUTTONBOX_END);
@@ -1847,4 +1877,22 @@ itip_view_get_buttons_sensitive (ItipView *view)
 	return priv->buttons_sensitive;
 }
 
+gboolean	
+itip_view_get_recur_check_state (ItipView *view)
+{
+	return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view->priv->recur_check));
+}
+
+void
+itip_view_set_show_recur_check (ItipView *view, gboolean show)
+{
+	g_return_if_fail (view != NULL);
+	g_return_if_fail (ITIP_IS_VIEW (view));	
 	
+	if (show)
+		gtk_widget_show (view->priv->recur_check);
+	else  {
+		gtk_widget_hide (view->priv->recur_check);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->priv->recur_check), FALSE);
+	}
+}
